function CallipegImport() 
{
	var dataFileURL = FileDialog.getOpenFileName(null, "Select data file");
    var dataFolderURL = dataFileURL;
	if(dataFileURL == null) return;

	var gVerticalFit = false
	var gFitPrompted = false
	
    if(dataFileURL.match("data")) dataFolderURL = dataFileURL.substring(0, Math.max(dataFileURL.lastIndexOf("/"), dataFileURL.lastIndexOf("\\")));
	var folder = new Dir(dataFolderURL);
	
	scene.beginUndoRedoAccum( "Import Callipeg project" );
	
	var stopmotion = preferences.getBool( "SP_CONSTANT_SEGMENT", false );
	preferences.setBool("SP_CONSTANT_SEGMENT", true);

	var files = folder.entryList( "*", DirSpec.Files, DirSpec.Name );
    var jsonObj;
	var scale = 0;

	// Checking all files to ensure we get the data file (in case user did choose another one)
	for( f=0; f < files.length; f++ ) 
    {
		var file = files[f];
		
		if (file.match("data")) 
        {
			var datafile = new File(folder.absPath + "/" + file);
			datafile.open( FileAccess.ReadOnly );
			const data = datafile.read();
			datafile.close();

			// Just parse JSON file, then you get a dictionnary easy to read
			jsonObj = JSON.parse( data );
        }
    }

	// var pixelscalex = scene.numberOfUnitsX() * scene.unitsAspectRatioX() / scene.unitsAspectRatioY() / scene.defaultResolutionX();
	// var pixelscaley = scene.numberOfUnitsY() / scene.defaultResolutionY();
	
	// The main compositing node
	var compNode = node.subNodeByName( node.root(), "Composite" );

	// Background color node
	var colorcard = node.add( node.root(), "Background color", "COLOR_CARD", -30, -30, 0 );
	node.link(colorcard, 0, compNode, node.numberOfInputPorts(compNode));
	node.setTextAttr(colorcard, "color.red", 0, jsonObj["bgColorR"]);
	node.setTextAttr(colorcard, "color.green", 0, jsonObj["bgColorG"]);
	node.setTextAttr(colorcard, "color.blue", 0, jsonObj["bgColorB"]);
	node.setTextAttr(colorcard, "color.alpha", 0, 255);

	scene.setFrameRate( jsonObj["fps"] );

	var minFrame = 999999
	var maxFrame = -1

	// Read all layers
	const totalLayerCount = jsonObj["layers"].length;
	var layerNodes = []
	for( layerIndex=0; layerIndex < totalLayerCount; layerIndex++ ) 
	{
		const layerInfoNode = jsonObj["layers"][totalLayerCount - layerIndex - 1]; // Reversed order

		// Names in Harmony can't have special caracters, so we remove  em all
		const layerNameAlphaNum = layerInfoNode["layer"]["name"].replace( /[^A-Za-z0-9]/g, "_" );

		if( layerInfoNode["type"] == "Drawing" )
		{
			var newLayer = AddlayerDrawing( layerNameAlphaNum, layerIndex );
			layerNodes[ layerNameAlphaNum ] = newLayer

			node.setTextAttr( newLayer, "opacity", 0, layerInfoNode["layer"]["opacity"] * 100 );
			node.setEnable( newLayer, layerInfoNode["layer"]["visibility"] );
			
			const minMax = ExtractInstancesFromNode( layerInfoNode["layer"]["instances"], newLayer, folder )
			if (minMax[0] < minFrame ) minFrame = minMax[0]
			if (minMax[1] > maxFrame ) maxFrame = minMax[1]

			// There are cycles
			if( layerInfoNode["layer"]["instanceCycles"].length > 0 )
			{
				for( cycleNodeIndex=0; cycleNodeIndex < layerInfoNode["layer"]["instanceCycles"].length; cycleNodeIndex++ )
				{
					var cycleNode = layerInfoNode["layer"]["instanceCycles"][cycleNodeIndex]
					const minMaxA = ExtractInstancesFromNode( cycleNode["instanceOriginals"], newLayer, folder )
					if (minMaxA[0] < minFrame ) minFrame = minMaxA[0]
					if (minMaxA[1] > maxFrame ) maxFrame = minMaxA[1]
	
					
					const minMaxB = ExtractInstancesFromNode( cycleNode["instanceRefs"], newLayer, folder )
					if (minMaxB[0] < minFrame ) minFrame = minMaxB[0]
					if (minMaxB[1] > maxFrame ) maxFrame = minMaxB[1]
				} 
			}
		}
		else if ( layerInfoNode["type"] == "Transformation" )
		{
			if( !gFitPrompted )
			{ GetFitOrientation() }

			var newPeg = node.add( node.root(), layerNameAlphaNum, "PEG", -(layerIndex*30), -(layerIndex*30), 0 );
			layerNodes[ layerNameAlphaNum ] = newPeg
			node.setTextAttr( newPeg, "POSITION.SEPARATE", 1, "Y" );
			node.setTextAttr( newPeg, "OFFSET.SEPARATE", 1, "Y" );
			node.setEnable( newLayer, layerInfoNode["layer"]["visibility"] );

			// Connect layers
			const childrenIndices = layerInfoNode["layer"]["childIndices"];
			for ( childIndex=0; childIndex < childrenIndices.length; childIndex++ )
			{
				const theLayerIndex = childrenIndices[childIndex]
				const theLayerName = jsonObj["layers"][ theLayerIndex ]["layer"]["name"].replace( /[^A-Za-z0-9]/g, "_" );
				node.link( newPeg, 0, layerNodes[ theLayerName ], 0 );

				CreateTransparency( layerNodes[ theLayerName ], layerInfoNode["layer"]["curves"], layerIndex )
			}

			// Read keyframes
			const curves = layerInfoNode["layer"]["curves"];
			BuildCurves( curves, newPeg, layerNameAlphaNum )

			// Pivot setup
			const pivotWeights = layerInfoNode["layer"]["pivotWeight"];
			const pivotPosPixel = [ scene.defaultResolutionX() * pivotWeights[0], scene.defaultResolutionY() * pivotWeights[1] ]
			const pivotPosField = FromPixels( pivotPosPixel[0], pivotPosPixel[1] )
			node.setTextAttr( newPeg, "POSITION.SEPARATE", 1, "Y" );
			node.setTextAttr( newPeg, "OFFSET.SEPARATE", 1, "Y" );
			node.setTextAttr( newPeg, "PIVOT.X", 1, pivotPosField.x );
			node.setTextAttr( newPeg, "PIVOT.Y", 1, pivotPosField.y );
		}
	}

	// Setting the mark out
	const movieLength = maxFrame - minFrame;
	const oldNbFrames = frame.numberOf();
	if (oldNbFrames < movieLength) frame.insert(frame.numberOf(), movieLength-oldNbFrames);
	else if(oldNbFrames > movieLength) frame.remove(frame.numberOf(), oldNbFrames-movieLength);

	
	preferences.setBool("SP_CONSTANT_SEGMENT", stopmotion);
	
	scene.endUndoRedoAccum();


// =================================================================================================================

	// I can't figure out how to deduce that using only FOV, or if there's another way, so i just ask the user
	function GetFitOrientation()
	{
		var dialog = new Dialog();
		dialog.title = "Select fov Orientation";
		var optionOne = new RadioButton();
		optionOne.text = "Horizontal";
		var optionTwo = new RadioButton();
		optionTwo.text = "Vertical";
		dialog.add( optionOne );
		dialog.newColumn();
		dialog.add( optionTwo );
		optionTwo.checked = true

		const rc = dialog.exec()
			
		if (rc)
		{ 
			gVerticalFit = optionTwo.checked
			gFitPrompted = true
		}

		return  rc;
	}


	function RadToDeg( value )
	{
		return  value * 180 / 3.14159265359
	}


	function BuildCurves( curvesJSON, pegNode, pegNodeName )
	{
		// Position X
		const columnPosX = pegNodeName + "_POSX";
		column.add( columnPosX, "BEZIER" );
		node.linkAttr( pegNode, "POSITION.X", columnPosX );

		if ( curvesJSON[0]["keys"].length > 0 )
		{
			for( keyIndex=0; keyIndex < curvesJSON[0]["keys"].length; keyIndex++ )
			{
				const keyJSON = curvesJSON[0]["keys"][keyIndex]
				const lTan = keyJSON["lTangent"];
				const rTan = keyJSON["rTangent"];
				const posX = keyJSON["time"] + 1; // Harmony starts at frame 1
				const posY = FromPixels( keyJSON["value"], 0 ).x - FromPixels( 0, 0 ).x; // This is a vector, meaning a B - A computation, with B being (0,0)
				const tanLConverted = FromPixels( lTan[1], 0 ).x - FromPixels( 0, 0 ).x; // Same with tangent vectors, we want the Y value of the curve, being X position to be in FieldUnits
				const tanRConverted = FromPixels( rTan[1], 0 ).x - FromPixels( 0, 0 ).x; // Same with tangent vectors, we want the Y value of the curve, being X position to be in FieldUnits
				func.setBezierPoint( columnPosX, posX, posY, posX + lTan[0], posY + tanLConverted, posX + rTan[0], posY + tanRConverted, false, "CORNER" )
			}
		}

		// Position Y
		const columnPosY = pegNodeName + "_POSY";
		column.add( columnPosY, "BEZIER" );
		node.linkAttr( pegNode, "POSITION.Y", columnPosY );

		if ( curvesJSON[1]["keys"].length > 0 )
		{
			for( keyIndex=0; keyIndex < curvesJSON[1]["keys"].length; keyIndex++ )
			{
				const keyJSON = curvesJSON[1]["keys"][keyIndex]
				const lTan = keyJSON["lTangent"];
				const rTan = keyJSON["rTangent"];
				const posX = keyJSON["time"] + 1; // Harmony starts at frame 1
				const posY = FromPixels( 0, keyJSON["value"] ).y - FromPixels( 0, 0 ).y; // This is a vector, meaning a B - A computation, with B being (0,0)
				const tanLConverted = FromPixels( 0, lTan[1] ).y - FromPixels( 0, 0 ).y; // Same with tangent vectors, we want the Y value of the curve, being Y position to be in FieldUnits
				const tanRConverted = FromPixels( 0, rTan[1] ).y - FromPixels( 0, 0 ).y; // Same with tangent vectors, we want the Y value of the curve, being Y position to be in FieldUnits
				func.setBezierPoint( columnPosY, posX, posY, posX + lTan[0], posY + tanLConverted, posX + rTan[0], posY + tanRConverted, false, "CORNER" )
			}
		}

		// Scale X
		const columnScaleXName = pegNodeName + "_ScaleX";
		column.add( columnScaleXName, "BEZIER" );
		node.linkAttr( pegNode, "SCALE.X", columnScaleXName );

		if ( curvesJSON[2]["keys"].length > 0 )
		{
			for( keyIndex=0; keyIndex < curvesJSON[2]["keys"].length; keyIndex++ )
			{
				const keyJSON = curvesJSON[2]["keys"][keyIndex]
				const lTan = keyJSON["lTangent"];
				const rTan = keyJSON["rTangent"];
				const posX = keyJSON["time"] + 1; // Harmony starts at frame 1
				const posY = keyJSON["value"];
				func.setBezierPoint( columnScaleXName, posX, posY, posX + lTan[0], posY + lTan[1], posX + rTan[0], posY + rTan[1], false, "CORNER" )
			}
		}

		// Scale Y
		const columnScaleYName = pegNodeName + "_ScaleY";
		column.add( columnScaleYName, "BEZIER" );
		node.linkAttr( pegNode, "SCALE.Y", columnScaleYName );

		if ( curvesJSON[3]["keys"].length > 0 )
		{
			for( keyIndex=0; keyIndex < curvesJSON[3]["keys"].length; keyIndex++ )
			{
				const keyJSON = curvesJSON[3]["keys"][keyIndex]
				const lTan = keyJSON["lTangent"];
				const rTan = keyJSON["rTangent"];
				const posX = keyJSON["time"] + 1; // Harmony starts at frame 1
				const posY = keyJSON["value"];
				func.setBezierPoint( columnScaleYName, posX, posY, posX + lTan[0], posY + lTan[1], posX + rTan[0], posY + rTan[1], false, "CORNER" )
			}
		}

		// Rotation
		const columnAngleZName = pegNodeName + "_AngleZ";
		column.add( columnAngleZName, "BEZIER" );
		node.linkAttr( pegNode, "ROTATION.ANGLEZ", columnAngleZName );

		if ( curvesJSON[4]["keys"].length > 0 )
		{
			for( keyIndex=0; keyIndex < curvesJSON[4]["keys"].length; keyIndex++ )
			{
				const keyJSON = curvesJSON[4]["keys"][keyIndex]
				const lTan = keyJSON["lTangent"];
				const rTan = keyJSON["rTangent"];
				const posX = keyJSON["time"] + 1; // Harmony starts at frame 1
				const posY = -RadToDeg(keyJSON["value"]); // Harmony is CCW, Callipeg is CW
				func.setBezierPoint( columnAngleZName, posX, posY, posX + lTan[0], posY + lTan[1], posX + rTan[0], posY + rTan[1], false, "CORNER" )
			}
		}
	}
		

	function AddlayerDrawing( layerName, num ) 
	{
		var elemId = element.add( layerName, "BW", scene.numberOfUnitsZ(), "SCAN", "TVG" );
		column.add( layerName, "DRAWING" );
		column.setElementIdOfDrawing( layerName, elemId );
		var vnode = node.add( node.root(), layerName, "READ", -(num*30), -(num*30), 0 );
		node.linkAttr( vnode, "DRAWING.ELEMENT", layerName );
		node.link( vnode, 0, compNode, node.numberOfInputPorts(compNode) );
		return vnode;
	}

	function ExtractInstancesFromNode( nodeToExtractFrom, layerToImportInto, folderPath )
	{
		var minFrame = 999999
		var maxFrame = -1

		for( instanceIndex=0; instanceIndex < nodeToExtractFrom.length; instanceIndex++ )
		{
			const instanceNode = nodeToExtractFrom[instanceIndex];
			const instanceNameAlphaNum = instanceNode["instanceID"].replace( /[^A-Za-z0-9]/g, "_" );
			instanceStartFrame = instanceNode["startingFrame"];
			const instanceEndFrame = instanceNode["endingFrame"];
			
			ImportDrawingInElementNode( folderPath.absPath + "/" + instanceNode["instanceID"] + ".png", { node: layerToImportInto }, instanceNameAlphaNum );
			
			for ( frm = instanceStartFrame; frm < instanceEndFrame; frm++ )
			{
				column.setEntry( layerNameAlphaNum, 1, frm+1, instanceNameAlphaNum );
			}
			
			if (instanceStartFrame < minFrame ) minFrame = instanceStartFrame
			if (instanceEndFrame > maxFrame ) maxFrame = instanceEndFrame
		}	

		return  [minFrame, maxFrame]
	}

	function GetConnectedElement( n ) 
	{
		const col = node.linkedColumn( n, "DRAWING.ELEMENT" );
		return column.getElementIdOfDrawing( col );
	}
	
	function GetImportItemScaleFactors(imageFilename) 
	{
		const r = CELIO.getInformation(imageFilename);
		if (!r) {
		  return { scaleX : 1, scaleY : 1 };
		}
		const resX = scene.defaultResolutionX() * 1.0;
		const resY = scene.defaultResolutionY() * 1.0;
		return {
		   scaleX : r.width / resX,
		   scaleY : r.height / resY
		};
	}
	
	function ImportDrawingInElementNode( imageURL, options, instanceName ) 
	{
		var vectorizeOptions = [];

		var additionalVectorizeOptions = [];
		additionalVectorizeOptions.push("-asBitmap");
		additionalVectorizeOptions.push("-no_color_art");
		additionalVectorizeOptions.push("-pixelPerModelUnit");

		const elemId = GetConnectedElement( options.node );

		var bitmapResolutionScaleFactor = 1.0;
		const pixelPerModelUnit = scene.defaultPixelPerModelUnitForBitmapLayers();
		const resolutionScaling = GetImportItemScaleFactors(imageURL);
		bitmapResolutionScaleFactor = resolutionScaling.scaleY;
		if(scale==0) scale = resolutionScaling.scaleY;
		additionalVectorizeOptions.push(bitmapResolutionScaleFactor * pixelPerModelUnit);
		additionalVectorizeOptions.push("-premultiply");
		additionalVectorizeOptions.push("straight");
		
		if (!Drawing.isExists(elemId, instanceName)) 
		{
			Drawing.create( elemId, instanceName, true );
		}
		const destinationFile = Drawing.filename( elemId, instanceName );
		var dk = Drawing.Key( elemId, instanceName );
		
		if (!QFile.exists(destinationFile))
		{
			dk = { filename : destinationFile };
		}
		
		const inputDesc = { "filename" : imageURL };
		DrawingTools.vectorize( inputDesc, dk, vectorizeOptions.join(" "), additionalVectorizeOptions.join(" ") );
	}


	function ToPixels( x, y )
	{
		const aspectRatio = scene.defaultResolutionX() / scene.defaultResolutionY();
		const fourThird = 4/3
		const ratioCorrectionX = aspectRatio / fourThird
	
		if (gVerticalFit)
		{
			const xMax = 12 * ratioCorrectionX
			const xNorm = (x + xMax) / xMax*2
			const yNorm = (y + 12) / 24
		
			const xPixel = scene.defaultResolutionX() * xNorm;
			const yPixel = scene.defaultResolutionY() - scene.defaultResolutionY() * yNorm;
	
			return  Point2d( xPixel, yPixel ) 
		}
		else 
		{
			const yMax = 12 / ratioCorrectionX
			const xNorm = (x + 12) / 24
			const yNorm = (y + yMax) / yMax*2
		
			const xPixel = scene.defaultResolutionX() * xNorm;
			const yPixel = scene.defaultResolutionY() - scene.defaultResolutionY() * yNorm;
	
			return  Point2d( xPixel, yPixel ) 
		}
	}
	
	function FromPixels( x, y )
	{
		const aspectRatio = scene.defaultResolutionX() / scene.defaultResolutionY();
		const fourThird = 4/3
		const ratioCorrectionX = aspectRatio / fourThird
	
		const xNorm = x / scene.defaultResolutionX()
		const yNorm = y / scene.defaultResolutionY()
	
		if (gVerticalFit)
		{
			const xMax = 12 * ratioCorrectionX
			const yMax = 12
	
			const fieldX = xMax * xNorm * 2 - xMax
			const fieldY = yMax * yNorm * 2 - yMax
	
			return  Point2d( fieldX, -fieldY ) 
		}
		else 
		{
			const xMax = 12
			const yMax = 12 / ratioCorrectionX
	
			const fieldX = xMax * xNorm * 2 - xMax
			const fieldY = yMax * yNorm * 2 - yMax
	
			return  Point2d( fieldX, -fieldY )
		}
	}

	function UnlinkNodeSrcFromNode( srcNode, theNodeToUnlinkFrom )
	{
		const nodeInputPortsCount = node.numberOfInputPorts(theNodeToUnlinkFrom)
		for( i=0; i < nodeInputPortsCount; i++ )
		{
			const srcNodeI = node.srcNode( theNodeToUnlinkFrom, i )
			if( srcNodeI == srcNode )
			{
				node.unlink( theNodeToUnlinkFrom, i )
				break
			}
		}
	}


	function CreateTransparency( forLayerNode, curveJSON, nodeUIIndex )
	{
		var fadeNode = node.add( node.root(), node.getName( forLayerNode ) + "_Transparency", "FADE", -(nodeUIIndex*32), -(nodeUIIndex*30), 0 );
		var compNode = node.subNodeByName( node.root(), "Composite" );

		UnlinkNodeSrcFromNode( forLayerNode, compNode )
		node.link( forLayerNode, 0, fadeNode, 0 );
		node.link( fadeNode, 0, compNode, node.numberOfInputPorts(compNode) );
	
		// OpacityCurve
		const columTrans = node.getName( forLayerNode ) + "_Transparency";
		column.add( columTrans, "BEZIER" );
		node.linkAttr( fadeNode, "TRANSPARENCY", columTrans );	

		if ( curveJSON[5]["keys"].length > 0 )
		{
			for( keyIndex=0; keyIndex < curveJSON[5]["keys"].length; keyIndex++ )
			{
				const keyJSON = curveJSON[5]["keys"][keyIndex]
				const lTan = keyJSON["lTangent"];
				const rTan = keyJSON["rTangent"];
				const posX = keyJSON["time"] + 1; // Harmony starts at frame 1
				const posY = 100 - keyJSON["value"] * 100; // Transparency is like: 0 means fully opaque, 100 means fully transparent, so it's inverted + 0-100 range
				func.setBezierPoint( columTrans, posX, posY, posX + lTan[0], posY + lTan[1], posX + rTan[0], posY + rTan[1], false, "CORNER" )
			}
		}
		
	}
}